from typing import List

from peracotta.peralog import logger


def parse_udevadm(file_content: str) -> List[dict]:
    devices = {}
    for line in file_content.splitlines():

        if not line.startswith("E:"):
            logger.error(f"Expected 'E:' at the beginning of line: {line}")
            logger.error(f"\n{file_content}")
            raise ValueError("Error while parsing udevadm output")

        key, value = line[3:].split("=")

        if not key[:14].startswith("MEMORY_DEVICE_"):
            logger.error(f"Expected 'MEMORY_DEVICE' at the beginning of key: {key}")
            logger.error(f"\n{file_content}")
            raise ValueError("Error while parsing udevadm output")

        device_id = key[14:].split("_")[0]  # could have taken a single char but this is more reliable

        if device_id not in devices:
            devices[device_id] = {}
        devices[device_id][key[15 + len(device_id) :]] = value.strip()

    dimms = []
    for device_id, device in devices.items():
        if device["SPEED_MTS"] == 666:
            device["SPEED_MTS"] = 667

        try:
            if device["SERIAL_NUMBER"][0:2] == "0x":
                sn = str(int(device["SERIAL_NUMBER"][2:], base=16))
            if any(c in "abcdefABCDEF" for c in device["SERIAL_NUMBER"]):
                sn = str(int(device["SERIAL_NUMBER"], base=16))
            else:
                sn = device["SERIAL_NUMBER"]
        except ValueError:
            logger.error(f"Error while parsing serial number: {device['SERIAL_NUMBER']}")
            logger.error(f"{file_content = }")
            continue

        if sn:
            dimm = {
                "type": "ram",
                "working": "yes",
                "ram-type": device["TYPE"].upper(),
                "frequency-hertz": int(device["SPEED_MTS"]) * 1000 * 1000,
                "capacity-byte": int(device["SIZE"]),
                "brand": device["MANUFACTURER"],
                "model": device["PART_NUMBER"],
                "sn": sn,
            }
            dimms.append(dimm)

    # MISSING ECC AND TIMINGS

    return dimms



from .read_decode_dimms import parse_decode_dimms
from .read_dmidecode import parse_case, parse_motherboard, parse_psu
from .read_lscpu import parse_lscpu
from .read_lspci_and_glxinfo import parse_lspci_and_glxinfo
from .read_smartctl import parse_smartctl
from .read_udevadm import parse_udevadm
from .windows_parser import parse_win_chassis_specs, parse_win_cpu_specs, parse_win_motherboard_specs, parse_win_ram_specs



"""
Python package `peracotta` provides a program intended to gather hardware data, display it and upload it to [T.A.R.A.L.L.O.](https://github.com/weee-open/tarallo).

The following documentation is intended for future mantainers and developers working on peracotta, and doesn't aim to be a clean easy-to-use user guide, although it may still prove useful in that sense.

Docs are automatically generated by [`pdoc3`](https://pdoc3.github.io/) using docstrings (following [PEP 257](https://peps.python.org/pep-0257/) conventions).

## Packaging
The packaging is done using [`pyproject.toml`](https://packaging.python.org/en/latest/guides/writing-pyproject-toml/) and [`setuptools`](https://setuptools.pypa.io/en/latest/)

Even if you're not familiar with this setup, simply reading the `pyproject.toml` file should be pretty self-explanatory.
Some interesting properties are:

 - **[project.scripts]** define commands that will be installed in the system (usually in ~/.local/bin, make sure it's in PATH) and what functions in the package they should call.
 - **[project.gui-scripts]** same thing but for commands that start a GUI.
 - **[tool.setuptools.package-data]** by default, only .py files are included in the built package. This property allows including useful data files such as assets and scripts. Find more info [here](https://setuptools.pypa.io/en/latest/userguide/datafiles.html#package-data).

## Styling
as a general rule for styling try to follow [Google Python Style Guide](https://google.github.io/styleguide/pyguide.html). Formatting is done with black (installed as a pre-commit hook).

## Contributing
You should **NEVER** push directly to the master branch.
Work on each feature/improvement/bugfix in a separate branch (usually in a fork in your personal github profile).

If two or more branches need to be merged together don't merge both in master, but first merge one into the other and resolve conflicts, then merge into master.

Only ever merge into master if all tests are passing.

When merging a branch, only use the 'rebase' option if there isn't more than a couple of commits. Otherwise, prefer 'squash'.

Be coincise and descriptive with your commits.
Avoid nonsensical or ambiguous descriptions, they will only make your life miserable when a new bug is found tomorrow and you have to trace back the root cause.

If you modify existing code make sure you keep the documentation up-to-date.
Mostly, the documentation is done through docstrings so it should be easy.
The docstrings should ideally be updated in the same commit that modified the interested code.

## Adding new features
Whenever a new feature is added make sure that you have added proper tests and documentation before merging
"""

import signal
import sys

from PyQt5 import QtWidgets

from . import peracruda
from .config import CONFIG
from .constants import VERSION
from .gui import GUI, errored, gui_excepthook
from .peralog import logdir, logger
from .crash_reporting import send_crash_notification


def parse_common_args():
    """
    Parse arguments common to both GUI and CLI version
    --version prints the current version and quits.
    --logs prints the path where logs are stored and quits.
    """
    if any([s in sys.argv for s in ["--version", "-v"]]):
        print(f"P.E.R.A.C.O.T.T.A. Version {VERSION}")
        exit(0)

    if "--logs" in sys.argv:
        print(f"P.E.R.A.C.O.T.T.A.'s logs are located in {logdir.as_posix()}")
        exit(0)


def main_gui():
    """Entrypoint for the GUI version."""
    parse_common_args()

    sys.excepthook = gui_excepthook
    signal.signal(signal.SIGINT, signal.SIG_DFL)  # makes CTRL+ C work

    app = QtWidgets.QApplication(sys.argv)

    # noinspection PyUnusedLocal
    window = GUI(app)
    app.exec_()

    try:
        if CONFIG["AUTOMATIC_REPORT_ERRORS"] and errored() and not send_crash_notification():
            logger.error("Couldn't upload crash log.")
    except Exception as e:
        logger.error(f"Error while sending crash notification: {e}")
        logger.exception(e)


def main_cli():
    """.. todo::Entrypoint for the CLI version"""
    print("Sorry, peracruda isn't implemented in v2 yet! Use the old one at https://github.com/WEEE-Open/peracotta/tree/v1.0.0")
    parse_common_args()
    # peracruda.__main() # Doesn't seem to prompt for sudo password and gets stuck


if __name__ == "__main__":
    try:
        if sys.argv[1] == "gui":
            main_gui()
        else:
            main_cli()
    except IndexError:
        main_cli()



"""
Peracotta is configurable in various ways.
The reccomended is with the file `~/.config/WEEE Open/peracotta/config.toml`.

Example config:
```toml
TARALLO_URL = "http://192.168.2.142:8080"
TARALLO_TOKEN = "yoLeCHmEhNNseN0BlG0s3A:ksfPYziGg7ebj0goT0Zc7pbmQEIYvZpRTIkwuscAM_k"
TARALLO_FEATURES_AUTO_DOWNLOAD = false

GENERATE_FILES_USE_SUDO = true
GENERATE_FILES_ASK_SUDO_PASSWORD = true

AUTOMATIC_REPORT_ERRORS = true
REPORT_URL = "http://127.0.0.1:9999"
```
This configuration can be overridden by `~/.config/WEEE Open/peracotta/.env`
and/or environment variables. If both are used, the latter take precedence.

For compatibility with the old M.I.S.O.'s martello.sh script, it's also possible to configure it by placing a .env file in the src directory. This file is only checked if peracotta is being launched directly from source instead of being installed.
For this reasons, there's a .env.example in the source code at the appropriate place
"""

import os
from pathlib import Path
from typing import Optional

import toml
from dotenv import load_dotenv

from .constants import basedir

HOME_DIR = Path().home()

CONF_DIR = HOME_DIR.joinpath(".config/WEEE Open/peracotta")
CONFIG = {}

keys = [
    "TARALLO_URL",
    "TARALLO_TOKEN",
    "TARALLO_FEATURES_AUTO_DOWNLOAD",
    "GENERATE_FILES_USE_SUDO",
    "GENERATE_FILES_ASK_SUDO_PASSWORD",
    "REPORT_URL",
    "AUTOMATIC_REPORT_ERRORS",
]


def parse_from_env(value: Optional[str]):
    if not value:
        return None

    trues = ["1", "true", "t", "", "yes", "y"]
    falses = ["0", "false", "f", "no", "n"]

    if value.lower() in trues:
        return True
    if value.lower() in falses:
        return False

    try:
        i = int(value)
        return i
    except ValueError:
        pass

    return value


# 1) src's .env, for compatibility with old M.I.S.O.
if isinstance(basedir, str):  # If the app is installed as a package basedir is a PosixPath object
    try:
        load_dotenv(basedir + "/../.env")  # doesn't override already defined variables
    except FileNotFoundError:
        pass


# 2) CONF_DIR's .env
try:
    load_dotenv(CONF_DIR.joinpath(".env"))
except FileNotFoundError:
    pass

for key in keys:
    CONFIG[key] = parse_from_env(os.environ.get(key))

# 3) CONF_DIR's toml
try:
    _toml_conf = toml.load(CONF_DIR.joinpath("config.toml"))
    for k in _toml_conf:
        if k not in CONFIG.keys() or CONFIG[k] is None:
            CONFIG[k] = _toml_conf[k]
except FileNotFoundError:
    pass

# 4) default toml
try:
    _toml_conf = toml.load(basedir.joinpath("config.toml"))
    for k in _toml_conf:
        if k not in CONFIG.keys() or CONFIG[k] is None:
            CONFIG[k] = _toml_conf[k]
except FileNotFoundError:
    pass



import os
import sys
from pathlib import Path

from loguru import logger

from .config import CONF_DIR

logdir = Path(CONF_DIR).joinpath("logs")
if not logdir.exists():
    os.makedirs(logdir)

try:
    DEBUG = sys.argv[1] == "DEBUG"
except IndexError:
    DEBUG = False

stdout_level = "DEBUG" if DEBUG else "WARNING"
file_level = "DEBUG" if DEBUG else "INFO"

log_format = "{time}\t{message}"
logger.remove()
logger.add(sys.stdout, format=log_format, level=stdout_level, colorize=True, backtrace=True, diagnose=True)
logger.add(logdir.joinpath("peracotta.log"), format=log_format, level=file_level)

# logger.info(f"{CONF_DIR = }")



from requests import Session

from .config import CONFIG
from .peralog import logdir

url = CONFIG["REPORT_URL"]


def send_crash_notification():
    with Session() as s:
        try:
            s.get(url, timeout=2)
        except TimeoutError:
            return False

        with open(logdir.joinpath("peracotta.log"), "r") as fs:
            res = s.post(url, files={"file": fs})

        if res.status_code == 200:
            return True
